% function generating of a random low frequency deformation field
% u is the input image to deform
% sigma is the standard deviation of the Gaussian kernel
% alpha is a rescaling factor to adjust the amplitude of the generated
% displacements in X and Y
% [X,Y] is the random deformation field generated
%
% Version 0.1, September 2017
% Author lara.raad@gmail.com

function [X,Y] = generate_random_deformation_field(H,W,sigma,...
                                                       max_def_x,max_def_y)

% initialization of the x and y displacement as white gaussian noises
wgn_x = randn(H,W);
wgn_y = randn(H,W);

% fft of the wgn
fft_wgn_x = fft2(wgn_x);
fft_wgn_y = fft2(wgn_y);

% gaussian filter
fft_filter = fft2(fspecial('gaussian',[H,W],sigma));

% filter in the Fourier domain
fft_wgn_x_filtered = fft_wgn_x.*(fft_filter);
fft_wgn_y_filtered = fft_wgn_y.*(fft_filter);

% random low frequency deformation field
X = real(ifft2(fft_wgn_x_filtered));
Y = real(ifft2(fft_wgn_y_filtered));

X = X/max(abs(X(:))); % limit to -1,1
Y = Y/max(abs(Y(:)));

X = X*max_def_x; % limit to max deformation (max_def)
Y = Y*max_def_y;