% this guide generator version takes into account the deformation on the
% boundary in order to avoid seeing the edges effect. for this the plankg
% is generated in a larger image and then cropped
%
% Version 0.4, september 2017
% Author lara.raad@gmail.com
%
% level sets method
% Creates random guide planks using level sets of a smooth function with
% up to two Gaussian bumps.
%
% Version 0.2, May 2017
% Author jlezama@fing.edu.uy

function [s1,g1,g2,s2,s3,s4,s5] = guide_generator(params)

% params = [H W sigma_blob sigma_disp max_disp blob_vert_pos...
% blob_hor_pos num_int blob_mult dilat_vert dilat_hor surf_decay...
% vert_disp_mult]

% check number of input parameters
errormsg = ['there should be 21 specified parameters  -- '...
'params = [H W sigma_blob sigma_disp max_disp blob_vert_pos '...
'blob_hor_pos num_int blob_mult dilat_vert dilat_hor surf_decay'...
'vert_disp_mult]'];
assert(length(params)==21,errormsg);

% get parameters
r = params(1); % ratio H/W
W1 = params(2); % width
H1 = r*W1; % height
% extension of H and W to have an output of size HxW
px = params(4)*1.1; % horizontal extension factor
py = params(5)*1.1; % vertical extension factor
H = H1+2*round(H1*py);
W = W1+2*round(W1*px);

sigma_displacement = round(params(3)*min(H1,W1));
max_displacement_x = params(4)*W1;
max_displacement_y = params(5)*H1;
blob_vertical_position =  params(6:7);
blob_horizontal_position = params(8:9);
num_intervals = params(10);
blob_multiplier = params(11:12);
dilation_length_vertical = max(1,round(params(13)*H1));
display(num2str(dilation_length_vertical));
dilation_length_horizontal = max(1,round(params(14)*W1));
display(num2str(dilation_length_horizontal));
surface_decay = params(15);

rho1 = params(16);
excentricity1 = params(17);
sig1 = params(18)*min(H1,W1);
mu11 = round(H1*blob_vertical_position(1)) + round(H1*py);
mu12 = round(W1*blob_horizontal_position(1)) + round(W1*px);

rho2 = params(19);
excentricity2 = params(20);
sig2 = params(21)*min(H1,W1);
mu21 = round(H1*blob_vertical_position(2)) + round(H1*py);
mu22 = round(W1*blob_horizontal_position(2)) + round(W1*px);

%gaussian parameters
%blob 1 covariance matrix
sig1_x = sig1;
sig1_y = sig1/excentricity1;

sig1_11 = sig1_x^2*cosd(rho1)^2+sig1_y^2*sind(rho1)^2;
sig1_22 = sig1_y^2*cosd(rho1)^2+sig1_x^2*sind(rho1)^2;
sig1_12 = (sig1_x^2-sig1_y^2)*cosd(rho1)*sind(rho1);
Sigma1 = [sig1_11 sig1_12;sig1_12 sig1_22];

%blob 2  covariance matrix
sig2_x = sig2;
sig2_y = sig2/excentricity2;

sig2_11 = sig2_x^2*cosd(rho2)^2+sig2_y^2*sind(rho2)^2;
sig2_22 = sig2_y^2*cosd(rho2)^2+sig2_x^2*sind(rho2)^2;
sig2_12 = (sig2_x^2-sig2_y^2)*cosd(rho2)*sind(rho2);
Sigma2 = [sig2_11 sig2_12;sig2_12 sig2_22];

% initalize surface
[X, ~] = meshgrid(1:W,1:H);
initial_surface = (1-normalize(abs(X-W/2).^surface_decay));
s1 = initial_surface;
% cut borders
s1 = cut_borders(s1,H1,W1,px,py);

% add two gaussian bumps
disp('generating gaussian blob 1')
% g1 = gaussian_blob(H,W, round(H*blob_vertical_position(1)),...
%     round(W*blob_horizontal_position(1)),...
%     Sigma1);
g1 = gaussian_blob(H,W,mu11,mu12,Sigma1);

disp('generating gaussian blob 2')
% g2 = gaussian_blob(H,W, round(H*blob_vertical_position(2)),...
%     round(W*blob_horizontal_position(2)),...
%     Sigma2);
g2 = gaussian_blob(H,W,mu21,mu22,Sigma2);

mysurface = initial_surface +g1*blob_multiplier(1) +g2*blob_multiplier(2);
mysurface = normalize(mysurface);
s2 = mysurface;
% cut borders
s2 = cut_borders(s2,H1,W1,px,py);

% add some random displacement
disp('generating random displacement')
%[vx, vy] = random_displacement(H, W, sigma_displacement,...
%                          max_displacement_x, max_displacement_y, 0, 0 ,0);
                      
[vx,vy] = generate_random_deformation_field(H,W,sigma_displacement,...
    max_displacement_x, max_displacement_y);

[X, Y] = meshgrid(1:W,1:H);
mysurface = interp2(mysurface, X + vx , Y + vy); % stronger on vertical
mysurface = normalize(mysurface);
mysurface(isnan(mysurface))=0;
s3 = mysurface;
% cut borders
s3 = cut_borders(s3,H1,W1,px,py);

% compute Level Lines
intervals = 0:1/num_intervals:1;
intervals = intervals.^2; % stretch intervals for larger gaps in cathedrals
intervals = sort(normalize(intervals + randn(size(intervals))/300));
L = length(intervals);

% colorize
red = zeros(H,W);
green = zeros(H,W);
blue = zeros(H,W);

light = [170,143,100]/255;
dark = [96 54 12]/255;

% light = [179, 150, 108
%     158, 133, 93
%     186, 158, 110
%     167, 136, 92
%     175, 150, 109
%     211, 181, 131]/255;
% 
% dark = [110, 78, 29
%     114, 81, 36
%     112, 82, 32
%     107, 75, 18
%     132, 96, 46
%     139, 110, 66]/255;

assert(size(dark,1)==size(light,1))
Ncolors = size(dark,1);

disp('painting intervals')
for i=1:L-1
    z = find(mysurface>=intervals(i) & mysurface<=intervals(i+1));
    
    if mod(i,2)
        colors = dark;
    else
        colors = light;
    end
    
    c = randi(Ncolors);
    red(z) = colors(c,1);
    green(z) = colors(c,2);
    blue(z) = colors(c,3);
end

new_colored = cat(3,red,green,blue);
s4 = new_colored;                      
% cut borders
s4 = cut_borders(s4,H1,W1,px,py);

disp('postprocessing gradients')
new_postprocessed = gradient_to_guide(normalize(imgradient(rgb2gray(...
                    new_colored))), dilation_length_vertical,...
                    dilation_length_horizontal, dark, light);
s5 = new_postprocessed;
% cut borders
s5 = cut_borders(s5,H1,W1,px,py);